#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <fcntl.h>
#include <unistd.h>
#include <time.h>
#include <signal.h>
#include <errno.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/select.h>
#include <sys/types.h>
#include <sys/time.h>
#include <linux/input.h>
#include "tslib.h"

struct	touchscreen_sample {
	int             x;
	int             y;
	unsigned int    pressure;
	int             x2;
	int             y2;
	int             nrfingers;
	int             gesture;
	struct timeval  tv;
};

struct	touchscreen_input {
	int             current_x;
	int             current_y;
	unsigned int    current_p;
	int             current_x2;
	int             current_y2;
	int             current_nrfingers;
	int             current_gesture;
};

struct touchscreen_input  temp_input; //Filled until sync event received

int			iHandle;
int			iPenPositionX=0;
int			iPenPositionY=0;
bool			iPenDown=false;
int			iPen2PositionX=0;
int			iPen2PositionY=0;
bool			iPen2Down=false;
int			iNrFingers=0;
int			iGesture=0;
unsigned long long 	iEventTime;

int	iSizeInputEvent;

int main(int argc, char* argv[])
{

    struct touchscreen_sample sample;
    struct input_event ev;

    bool stateChanged = false;
    bool touchScreenEventAvailable = false;
    int xtemp,ytemp;
    unsigned long long seconds;
    unsigned long long microSeconds;

    iSizeInputEvent=sizeof(input_event);

    iHandle = open("/dev/input/event0", O_NONBLOCK);

    if (iHandle)
    {
      // make read() block until data is available
      int flags = fcntl(iHandle, F_GETFL, 0);
      if (flags >= 0)
      {
        fcntl(iHandle, F_SETFL, flags & ~O_NONBLOCK);
      }
      printf("TouchScreen: Switched On\n");
    }else
    {
      return -1; 
    }

    init_linear_filter();

    printf("\n                      Time in Secs     PenDown     X1     Y1     Pen2Down     X2     Y2\n");


	while(1){

    touchScreenEventAvailable = false; 
    stateChanged = false;


    int nr = read(iHandle, &ev, iSizeInputEvent);

    if ( nr >= iSizeInputEvent ) {
	
	switch (ev.type) {

                        case EV_SYN:
				printf("\n");
                                /* Fill out a new complete event */
                                sample.x = temp_input.current_x;
                                sample.y = temp_input.current_y;
                                sample.pressure = temp_input.current_p;
				sample.x2 = temp_input.current_x2;
				sample.y2 = temp_input.current_y2;
				sample.nrfingers = temp_input.current_nrfingers;
				sample.gesture = temp_input.current_gesture;
                                sample.tv = ev.time;
                                touchScreenEventAvailable = true;
                                break;
                        case EV_ABS:
                                switch (ev.code) {
                                case ABS_X:
                                        temp_input.current_x = ev.value;
                                        break;
                                case ABS_Y:
                                        temp_input.current_y = ev.value;
                                        break;
                                case ABS_PRESSURE:
                                        temp_input.current_p = ev.value;
                                        break;
				case ABS_FINGER2_X:
					temp_input.current_x2 = ev.value;
					break;
				case ABS_FINGER2_Y:
					temp_input.current_y2 = ev.value;
					break;
				case ABS_FINGERS:
					temp_input.current_nrfingers = ev.value;
                                        break;
                                case ABS_GESTURE:
                                        temp_input.current_gesture = ev.value;
                                        break;
                                }
                                break;
                        }
    }

    if (touchScreenEventAvailable)
    {

 	const unsigned long long timerTicksPerSecond = 1000; // The value is #define'd, but we need it to be type-safe
 	const unsigned long long microSecTomilliSecDivisor = 1000;
 
 	if ( sample.tv.tv_sec==0 )
 	{
 		gettimeofday(&sample.tv, NULL);
 	}
 
	seconds =      static_cast<unsigned long long>(sample.tv.tv_sec);
        microSeconds = timerTicksPerSecond * static_cast<unsigned long long>(sample.tv.tv_usec) / microSecTomilliSecDivisor;
 	//iEventTime = seconds + microSeconds;

	printf(
        "Before Filtering:   %10lld.%06lld     %d        %03d     %03d      %d         %03d     %03d\n",
	seconds,
	microSeconds,
        (sample.nrfingers > 0),
        sample.x,
        sample.y,
	(sample.nrfingers > 1),
        sample.x2,
        sample.y2
        );

        xtemp = sample.x; ytemp = sample.y;

        linear_filter(xtemp, ytemp, &sample.x, &sample.y);

        // Same for finger2

	xtemp = sample.x2; ytemp = sample.y2;

	linear_filter(xtemp, ytemp, &sample.x2, &sample.y2);

        int newPenPositionX = sample.x;
        int newPenPositionY = sample.y;
        int newPen2PositionX = sample.x2;
        int newPen2PositionY = sample.y2;
        int newNrFingers = sample.nrfingers;
 
        //Ignore finger position on finger release event, instead take the last position
        //of the finger on the screen
        if ( (iNrFingers == 2) && (newNrFingers == 1) ) {
 		newPen2PositionX = iPen2PositionX;
 		newPen2PositionY = iPen2PositionY;
 	} else if ( (iNrFingers == 1) && (newNrFingers == 0) ) {
 		newPenPositionX = iPenPositionX;
 		newPenPositionY = iPenPositionY;
 	}
 
	// The quantity of pens down changed
 	if ( iNrFingers != newNrFingers ) {
 		iPenDown = ( newNrFingers > 0);
 		iPen2Down = ( newNrFingers > 1);
 		iNrFingers = newNrFingers;
 		stateChanged=true;
 	}

       // The position of pen 1 changed
       if ((iPenPositionX != newPenPositionX) || (iPenPositionY != newPenPositionY))
       {
 		iPenPositionX = newPenPositionX;
 		iPenPositionY = newPenPositionY;
 		stateChanged = true;
       }

       // The position of pen 2 changed
       if ((iPen2PositionX != newPen2PositionX) || (iPen2PositionY != newPen2PositionY))
       {
 		iPen2PositionX = newPen2PositionX;
 		iPen2PositionY = newPen2PositionY;
 		stateChanged = true;
       }

    }
 
    if (stateChanged)
    {
 	const unsigned long long timerTicksPerSecond = 1000; // The value is #define'd, but we need it to be type-safe
 	const unsigned long long microSecTomilliSecDivisor = 1000;
 
 	if ( sample.tv.tv_sec==0 )
 	{
 		gettimeofday(&sample.tv, NULL);
 	}
 
	seconds =      static_cast<unsigned long long>(sample.tv.tv_sec);
        microSeconds = timerTicksPerSecond * static_cast<unsigned long long>(sample.tv.tv_usec) / microSecTomilliSecDivisor;
	printf(
         "After  Filtering:   %10lld.%06lld     %d        %03d     %03d      %d         %03d     %03d\n",
         seconds,
         microSeconds,
         iPenDown,
         iPenPositionX,
         iPenPositionY,
         iPen2Down,
         iPen2PositionX,
         iPen2PositionY
         );
    }

	}

    close(iHandle);

    return stateChanged;

  }


